/*
 * inet.c
 *
 * Josef Kufner <jkufner@seznam.cz>
 * 
 * Knihovna funkci pro komunikaci po siti
 */


#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define DEBUG 		10
#include "debug.h"

#include "inet.h"


/*
 * Zakladni funkce pro komunikaci 
 */

/* Serverove funkce */
int inet_server_connect(const char *host, const int port, 
		const char *accept_host)
{
	int server_sockfd, client_sockfd;
	int server_len, client_len;
	struct sockaddr_in server_address;
	struct sockaddr_in client_address;
	
	/* Tvorba socketu */
	if ((server_sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		DBG_MSG(3, "inet_server_connect: socket failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "Socket ok.");	

	/* Konfigurace socketu */
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = inet_addr(host);
	server_address.sin_port = htons(port);
	server_len = sizeof(server_address);

	/* Pripojeni socketu */
	if (bind(server_sockfd, (struct sockaddr *) &server_address, server_len) == -1) {
		DBG_MSG(3, "inet_server_connect: bind failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_server_connect: Bind ok.");	
		

	/* Tvorba fronty */
	if (listen(server_sockfd, 5) == -1) {
		DBG_MSG(3, "inet_server_connect: listen failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_server_connect: listen ok.");	

	/* Cekani na klienta */	
	
	client_len = sizeof(client_address);
	if ((client_sockfd = accept(server_sockfd,
			(struct sockaddr *) &client_address, &client_len)) == -1) 
	{
		DBG_MSG(3, "inet_server_connect: accept failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_server_connect: Conected.");	

	/* Kontrola klienta */
	/* ... */

	return 0;
}

int inet_server_close(const int sockfd) {
	close(sockfd);
	return 0;
}


/* Klientske funkce */
int inet_client_connect(const char *host, const int port) {
	int sockfd;
	int len;
	struct sockaddr_in address;
	int result;

	/* Vytvoreni socketu */
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		DBG_MSG(3, "inet_client_connect: socket failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_client_connect: Socket ok.");

	/* Konfigurace socketu */
	address.sin_family = AF_INET;
	address.sin_addr.s_addr = inet_addr(host);
	address.sin_port = htons(port);
	len = sizeof(address);

	/* Pripojeni k serveru */
	if((result = connect(sockfd, (struct sockaddr *) &address, len)) == -1) {
		DBG_MSG(3, "inet_client_connect: connect failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_client_connect: Conected.");

	return result == -1 ? errno : sockfd;
}

int inet_client_close(const int sockfd) {
	close(sockfd);
}

/* Obecne funkce */
int inet_send(const int sockfd, const char *string) {
	DBG_MSG(6, "inet_send...");
	return write(sockfd, string, strlen(string) + 1);
}

int inet_receive(const int sockfd, char *string) {
	char ch, str[256];
	int i = 0;

	DBG_MSG(6, "inet_receive...");	
	do { 
		if (read(sockfd, &ch, 1) == 0) {
			return -1;
		}

		str[i] = ch;
		i++;
	
	} while ( ch != '\0');

	str[i] = '\0';

	DBG_MSG(6, "received: \"%s\"", str)
	
	string = str;
	
	return i;
	
}


