/*
 * inet.h
 *
 * Josef Kufner <jkufner@seznam.cz>
 * 
 * Knihovna funkci pro komunikaci po siti
 */

#ifndef INET_API_LIBRARY
#define INET_API_LIBRARY

/*
 * Zakladni funkce pro komunikaci 
 */

/* Serverove funkce */
int inet_server_connect(const char *host, const int port, 
		const char* accept_host);
	/* Vytvory srverovy socket se vsim vsudy. A Cekani na 1. klienta.
	 * 
	 *  Parametry:
	 *   host - retezec s ip adresou pocitace
	 *   port - cislo portu
	 *   accept_host - retezec s adresou povoleneho klienta
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		sockfd
	 */

int inet_server_close(const int sockfd);
	/* Zlikviduje serverovy socket se vsim vsudy.
	 *
	 * Parametry:
	 *   sockfd - deskriptor socketu
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		0
	 */


/* Klientske funkce */
int inet_client_connect(const char *host, const int port);
	/* Vytvori klientsky socket a pripoji se k serveru.
	 *
	 * Parametry:
	 *   host - cilovy server
	 *   port - port k pripojeni
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		sockfd
	 */

int inet_client_close(const int sockfd);
	/* Likvidace klienta.
	 *
	 * Parametry:
	 *   sockfd - deskriptor socketu
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		0
	 */

/* Obecne funkce */
int inet_send(const int sockfd, const char *string);
	/* Poslani dat na socket
	 *
	 * Parametry:
	 *   sockfd - deskryptor socketu
	 *   string - data k poslani
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		Pocet poslanych znaku
	 */
int inet_receive(const int sockfd, char *string);
	/* Prijem dat ze socketu.
	 *
	 * Parametry:
	 *   sockfd - deskryptor socketu
	 *   string - pointer, kam budou ulozena prijata data
	 *
	 * Vraci:
	 *   Pri chybe:		- errno
	 *   Normalne:		Pocet prijatych znaku
	 */

#endif
