/******************************************************************************
 * 
 * 		Piskvorky - NetGate
 * 		netgate.c
 *
 * 		Josef Kufner <jkufner@seznam.cz>
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#define DEBUG		6
#include "debug.h"

#define FD_STDIN	0
#define FD_STDOUT	1
#define FD_STDERR	2

char adr[256] = "127.0.0.1";
long int port = 5544;
int is_server = 0;
int accept_all = 0;

unsigned int verbose = 0;
unsigned int debug_print = 0;

int done = 0;

int fd_server, fd_client;

int Init_Server (void);
int Init_Client (void);
int Close_Server (void);
int Close_Client (void);
void on_Signal (int sig);
void on_Exit (void);

	
int Init_Server (void) {
	int server_sockfd, client_sockfd;
	int server_len, client_len;
	struct sockaddr_in server_address;
	struct sockaddr_in client_address;
	
	/* Tvorba socketu */
	if ((server_sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		DBG_MSG(3, "Init_Server: socket failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "Init_Server: Socket ok.");	

	/* Konfigurace socketu */
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = inet_addr(adr);
	server_address.sin_port = htons(port);
	server_len = sizeof(server_address);

	/* Pripojeni socketu */
	if (bind(server_sockfd, (struct sockaddr *) &server_address, server_len) == -1) {
		DBG_MSG(3, "Init_Server: bind failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "Init_Server: Bind ok.");	
		

	/* Tvorba fronty */
	if (listen(server_sockfd, 5) == -1) {
		DBG_MSG(3, "Init_Server: listen failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "Init_Server: listen ok.");	

	/* Cekani na klienta */	
	
	client_len = sizeof(client_address);
	if ((client_sockfd = accept(server_sockfd,
			(struct sockaddr *) &client_address, &client_len)) == -1) 
	{
		DBG_MSG(3, "Init_Server: accept failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "Init_Server: Conected.");	

	/* Kontrola klienta */
	/* ... */

	/* Nastaveni globalnich promennych */
	fd_server = server_sockfd;
	fd_client = client_sockfd;
	
	return 0;
}

int Init_Client (void) {
	int sockfd;
	int len;
	struct sockaddr_in address;
	int result;

	/* Vytvoreni socketu */
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		DBG_MSG(3, "inet_client_connect: socket failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_client_connect: Socket ok.");

	/* Konfigurace socketu */
	address.sin_family = AF_INET;
	address.sin_addr.s_addr = inet_addr(adr);
	address.sin_port = htons(port);
	len = sizeof(address);

	/* Pripojeni k serveru */
	if((result = connect(sockfd, (struct sockaddr *) &address, len)) == -1) {
		DBG_MSG(3, "inet_client_connect: connect failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	DBG_MSG(6, "inet_client_connect: Conected.");

	/* Nastaveni globalnich promennych */
	fd_client = sockfd;

	return 0;
}

int Close_Server (void) {
	if (fd_client) {
		Close_Client();
	}
	if (shutdown(fd_server, 1)) {
		fprintf(stderr, "NetGate: Nelze ukoncit spojeni: (%d) %s\n",
				errno, strerror(errno));
		exit(1);
	}
	fd_server = 0;
	return 0;
}

int Close_Client (void) {
	if (shutdown(fd_client, 1)) {
		fprintf(stderr, "NetGate: Nelze ukoncit spojeni: (%d) %s\n",
				errno, strerror(errno));
		exit(1);
	}
	fd_client = 0;
	return 0;
}

void on_Signal(int sig) {
	/* Bude volana fn. on_Exit */
	exit(0);
}

void on_Exit (void) {
	if (is_server) {
		Close_Server();
	} else {
//		Close_Client();
	}	
}

int main (int argc, char *argv[]) {
	int opt, ret;
	unsigned char byte;
	fd_set input_fd;

	DBG_MSG(3, "NetGate Starting...");
		
 	/* Dekodovani parametru  */
	while ((opt = getopt(argc, argv, "a:p:scAvd?h")) != -1) {
		switch(opt) {

		case 'a': /* IP Adresa */
			strcpy(adr, optarg);
			break;

		case 'p': /* Port */
			sscanf(optarg, "%ld", &port);
			break;

		case 's': /* Je Server */
			is_server = 1;
			break;

		case 'c': /* Je client */
			is_server = 0;
			break;

		case 'A': /* Prijmout vsechny klienty */
			accept_all = 1;
			break;

		case 'v': /* verbose */
			verbose++;
			break;
			
	
		case 'd': /* Ladici vypisy */
			debug_print = 1;
			break;
		case '?': /* Napoveda */
		case 'h':
		default:
			fprintf(stderr, "Pouziti: piskv-netgate <options>\n"
					"\t-h ... Vypise tuto napovedu\n"
					"\t-p ... Nastaveni komunikacniho portu"
					"\t-a ... IP Adresa Serveru, pripadne zadouciho klienta"
					"\t-c ... budu client"
					"\t-s ... budu server"
					"\t-A ... Prijmout kohokoliv (jinak jen adresy dane arg. '-a')"
					"\t-v ... verbose (ukecanost)"
					"\t-d ... Debug (volitelne)\n"
					"\n"
					"Tento program lze it a modifikovat v souladu\n"
					"s podmnkami licence GNU Public Licence.\n"
					"Pouvn tohoto programu je BEZ JAKKOLIV ZRUKY\n"
					"a vhradn na VLASTN NEBEZPE.\n"
					"\n");
			return 1;
			break;
		}
	}

	signal(SIGINT, on_Signal);
	signal(SIGTERM, on_Signal);
	signal(SIGABRT, on_Signal);
	atexit(on_Exit);

	/* Navazani spojeni */
	if (is_server) {
		if (Init_Server()) {
			/* Error */
			fprintf(stderr, "NetGate: Nemohu navazat spojeni: (%d) %s\n", 
					errno, strerror(errno));
			exit(2);
		}
	} else {
		if (Init_Client()) {
			/* Error */
			fprintf(stderr, "NetGate: Nemohu navazat spojeni! (%d) %s\n", 
					errno, strerror(errno));
			exit(2);
		}
	}

	/* Prenos dat */	
	while (!done) {
		/* Nastaveni pro select */
		FD_ZERO(&input_fd);
		FD_SET(FD_STDIN, &input_fd);
		FD_SET(fd_client, &input_fd);
	
		ret = select(FD_SETSIZE, &input_fd, NULL, NULL, NULL);
		DBG_MSG(7, "Select return: %d. errno = %d - \"%s\"", ret, errno, strerror(errno));

		if (ret < 0) {
			/* Chyba */
			fprintf(stderr, "Select: Error.\n");
			fflush(stderr);
			exit(1);
			break;
		}
		if (ret == 0) {
			/* Nic se nedeje */
			continue;
		}
		
		/* Mame data */
		if (FD_ISSET(FD_STDIN, &input_fd)) {
			/* stdin -> inet */
			ret = read(FD_STDIN, &byte, 1);
			if (ret <= 0) {
				DBG_MSG(3, "stdin->inet: error: %d - \"%s\"",
						errno, strerror(errno));
				done = 1;
			}
			
			ret = write(fd_client, &byte, 1);
			if (ret <= 0) {
				DBG_MSG(3, "stdin->inet: error: %d - \"%s\"",
						errno, strerror(errno));
				done = 1;
			}
		} else {
			/* inet -> stdout */
			ret = read(fd_client, &byte, 1);
			if (ret <= 0) {
				DBG_MSG(3, "inet->stdout: error: %d - \"%s\"",
						errno, strerror(errno));
				done = 1;
			}
			
			ret = write(FD_STDOUT, &byte, 1);
			if (ret <= 0) {
				DBG_MSG(3, "inet->stdout: error: %d - \"%s\"",
						errno, strerror(errno));
				done = 1;
			}
		}
	}

	/* Ukonceni spojeni probehne pri ukonceni programu (fn. atexit) */
	return 0;
}

